
//// UN-TWEAKABLES - AUTOMATICALLY-TRACKED TRANSFORMS ////////////////

float4x4 WorldITXf : WorldInverseTranspose < string UIWidget="None"; >;
float4x4 WvpXf : WorldViewProjection < string UIWidget="None"; >;
float4x4 WorldXf : World < string UIWidget="None"; >;
float4x4 ViewIXf : ViewInverse < string UIWidget="None"; >;

//// TWEAKABLE PARAMETERS ////////////////////

/// Point Lamp 0 ////////////
float3 Lamp0Pos : Position <
    string Object = "PointLight0";
    string UIName =  "Lamp 0 Position";
    string Space = "World";
> = {-0.5f,2.0f,1.25f};

float3 Lamp0Color : Specular <
    string UIName =  "Lamp 0";
    string Object = "Pointlight0";
    string UIWidget = "Color";
> = {1.0f,1.0f,1.0f};

// Ambient Light
float3 AmbiColor : Ambient <
    string UIName =  "Ambient Light";
    string UIWidget = "Color";
> = {0.07f,0.07f,0.07f};

//////// COLOR & TEXTURE /////////////////////

texture ColorTexture : DIFFUSE <
    string ResourceName = "brick_color.dds";
    string UIName =  "Decal Texture";
    string ResourceType = "2D";
>;
sampler2D ColorSampler = sampler_state {
    Texture = <ColorTexture>;
    FILTER = MIN_MAG_MIP_LINEAR;
    AddressU = Wrap;
    AddressV = Wrap;
};  

texture NormalTexture : NORMAL <
    string ResourceName = "brick_normals.dds";
    string UIName =  "Tangent Space Normal Texture";
    string ResourceType = "2D";
>;
sampler2D NormalMapSampler = sampler_state {
    Texture = <NormalTexture>;
    FILTER = MIN_MAG_MIP_LINEAR;
    AddressU = Wrap;
    AddressV = Wrap;
};  

texture GlossTexture : SPECULAR <
    string ResourceName = "brick_gloss.dds";
    string UIName =  "Gloss Texture";
    string ResourceType = "2D";
>;
sampler2D GlossMapSampler = sampler_state {
    Texture = <GlossTexture>;
    FILTER = MIN_MAG_MIP_LINEAR;
    AddressU = Wrap;
    AddressV = Wrap;
};  

texture SphereMapTexture : SPECULAR <
    string ResourceName = "uffizi_probe.png";
    string UIName =  "Sphere Map";
    string ResourceType = "2D";
>;
sampler2D SphereMapSampler = sampler_state {
    Texture = <SphereMapTexture>;
    FILTER = MIN_MAG_MIP_LINEAR;
    AddressU = Wrap;
    AddressV = Wrap;
};  

texture ParabMap01Texture : SPECULAR <
    string ResourceName = "parab01.png";
    string UIName =  "Sphere Map";
    string ResourceType = "2D";
>;
sampler2D ParabMap01Sampler = sampler_state {
    Texture = <ParabMap01Texture>;
    FILTER = MIN_MAG_MIP_LINEAR;
    AddressU = Wrap;
    AddressV = Wrap;
}; 

texture ParabMap02Texture : SPECULAR <
    string ResourceName = "parab02.png";
    string UIName =  "Sphere Map";
    string ResourceType = "2D";
>;
sampler2D ParabMap02Sampler = sampler_state {
    Texture = <ParabMap01Texture>;
    FILTER = MIN_MAG_MIP_LINEAR;
    AddressU = Wrap;
    AddressV = Wrap;
}; 
texture EnvMap : ENVIRONMENT <
    string ResourceName = "EnvironmentMap.dds";
    string UIName =  "Environment";
    string ResourceType = "Cube";
>;

samplerCUBE EnvMapSampler = sampler_state 
{
    texture = <EnvMap>;
    FILTER = MIN_MAG_MIP_LINEAR;
    AddressU  = Clamp;        
    AddressV  = Clamp;
    AddressW  = Clamp;
};

// shared shadow mapping supported in Cg version

//////// CONNECTOR DATA STRUCTURES ///////////

/* data from application vertex buffer */
struct appdata {
    float3 Position	: POSITION;
    float4 UV		: TEXCOORD0;
    float4 Normal	: NORMAL;
    float4 Tangent	: TANGENT0;
};

/* data passed from vertex shader to pixel shader */
struct vertexOutput {
    float4 HPosition	: POSITION;
    float2 UV		    : TEXCOORD0;
    // The following values are passed in "World" coordinates since
    //   it tends to be the most flexible and easy for handling
    //   reflections, sky lighting, and other "global" effects.
    float3 LightDir      : TEXCOORD1;
    float3 WorldNormal	 : TEXCOORD2;
    float3 WorldTangent	 : TEXCOORD3;
    float3 WorldBinormal : TEXCOORD4;
    float3 WorldView	 : TEXCOORD5;
};
 
///////// VERTEX SHADING /////////////////////

/*********** Generic Vertex Shader ******/

vertexOutput std_VS(appdata IN) {
    vertexOutput OUT = (vertexOutput)0;
	
    OUT.WorldNormal   = mul(IN.Normal,WorldITXf).xyz;
    OUT.WorldTangent  = mul(IN.Tangent,WorldITXf).xyz;
  	// Binormale in World Space
	OUT.WorldBinormal    = cross(OUT.WorldNormal,OUT.WorldTangent).xyz;

    float4 Position = float4(IN.Position.xyz,1);
    float3 PosWorld = mul(Position,WorldXf).xyz;
	
    OUT.LightDir = float4(Lamp0Pos,0);
    OUT.UV = float2(IN.UV.x,(1.0-IN.UV.y));
	
    OUT.WorldView = normalize(ViewIXf[3].xyz - PosWorld);
    OUT.HPosition = mul(Position,WvpXf);
    return OUT;
}

///////// PIXEL SHADING //////////////////////

float4 cubeMap_PS(vertexOutput IN) : COLOR {
	
   	// compute matrix for tangent space transformation 
	float3x3 matWorldToTangentSpace;
	matWorldToTangentSpace[0] = normalize(IN.WorldTangent);
	matWorldToTangentSpace[1] = normalize(IN.WorldBinormal);
	matWorldToTangentSpace[2] = normalize(IN.WorldNormal);
	
	float3 decalColor = tex2D(ColorSampler, IN.UV).xyz;
    float3 normalMap = tex2D(NormalMapSampler, IN.UV).xyz * 2.0 - 1.0;
	float3 glossMap = tex2D(GlossMapSampler, IN.UV).xyz;
	 
	// compute parameters N,L,V,H 
    float3 Nn = normalize(mul(normalMap,matWorldToTangentSpace));
    float3 Ln = normalize(IN.LightDir);
	float3 Vn = normalize(IN.WorldView);
	float3 Hn = normalize(Vn+Ln);
	float3 Rn = -reflect(Vn,Nn);
	
    float ldn = max(dot(Ln,Nn),0.0);
    float hdn = max(dot(Hn,Nn),0.0);
	
    float3 result = decalColor * ldn * Lamp0Color;
    result += glossMap * 0.3* pow(hdn,200);
	
    return float4(result,1);
}

///////// PIXEL SHADING //////////////////////

float4 sphereMap_PS(vertexOutput IN) : COLOR {
	
   	// compute matrix for tangent space transformation 
	float3x3 matWorldToTangentSpace;
	matWorldToTangentSpace[0] = normalize(IN.WorldTangent);
	matWorldToTangentSpace[1] = normalize(IN.WorldBinormal);
	matWorldToTangentSpace[2] = normalize(IN.WorldNormal);
	
	float3 decalColor = tex2D(ColorSampler, IN.UV).xyz;
    float3 normalMap = tex2D(NormalMapSampler, IN.UV).xyz * 2.0 - 1.0;
	float3 glossMap = tex2D(GlossMapSampler, IN.UV).xyz;
	 
	// compute parameters N,L,V,H 
    float3 Nn = normalize(mul(normalMap,matWorldToTangentSpace));
    float3 Ln = normalize(IN.LightDir);
	float3 Vn = normalize(IN.WorldView);
	float3 Hn = normalize(Vn+Ln);
	float3 Rn = -reflect(Vn,Nn);
	
    float ldn = max(dot(Ln,Nn),0.0);
    float hdn = max(dot(Hn,Nn),0.0);
	
    float3 result = decalColor * ldn * Lamp0Color;
    result += glossMap * 0.3* pow(hdn,200);
	
    return float4(result,1);
}

///////// PIXEL SHADING //////////////////////

float4 parabMap_PS(vertexOutput IN) : COLOR {
	
   	// compute matrix for tangent space transformation 
	float3x3 matWorldToTangentSpace;
	matWorldToTangentSpace[0] = normalize(IN.WorldTangent);
	matWorldToTangentSpace[1] = normalize(IN.WorldBinormal);
	matWorldToTangentSpace[2] = normalize(IN.WorldNormal);
	
	float3 decalColor = tex2D(ColorSampler, IN.UV).xyz;
    float3 normalMap = tex2D(NormalMapSampler, IN.UV).xyz * 2.0 - 1.0;
	float3 glossMap = tex2D(GlossMapSampler, IN.UV).xyz;
	 
	// compute parameters N,L,V,H 
    float3 Nn = normalize(mul(normalMap,matWorldToTangentSpace));
    float3 Ln = normalize(IN.LightDir);
	float3 Vn = normalize(IN.WorldView);
	float3 Hn = normalize(Vn+Ln);
	float3 Rn = -reflect(Vn,Nn);
		
    float ldn = max(dot(Ln,Nn),0.0);
    float hdn = max(dot(Hn,Nn),0.0);
	
    float3 result = decalColor * ldn * Lamp0Color;
    result += glossMap * 0.3* pow(hdn,200);
	
    return float4(result,1);
}

///// TECHNIQUES /////////////////////////////

RasterizerState DisableCulling
{
    CullMode = NONE;
};

DepthStencilState DepthEnabling
{
	DepthEnable = TRUE;
};

BlendState DisableBlend
{
	BlendEnable[0] = FALSE;
};

technique10 CubeMap10 <
	string Script = "Pass=p0;";
> {
    pass p0 <
	string Script = "Draw=geometry;";
    > {
        SetVertexShader( CompileShader( vs_4_0, std_VS() ) );
        SetGeometryShader( NULL );
        SetPixelShader( CompileShader( ps_4_0, cubeMap_PS() ) );
                
        SetRasterizerState(DisableCulling);       
		SetDepthStencilState(DepthEnabling, 0);
		SetBlendState(DisableBlend, float4( 0.0f, 0.0f, 0.0f, 0.0f ), 0xFFFFFFFF);
    }
}

technique10 SphereMap10 <
	string Script = "Pass=p0;";
> {
    pass p0 <
	string Script = "Draw=geometry;";
    > {
        SetVertexShader( CompileShader( vs_4_0, std_VS() ) );
        SetGeometryShader( NULL );
        SetPixelShader( CompileShader( ps_4_0, sphereMap_PS() ) );
                
        SetRasterizerState(DisableCulling);       
		SetDepthStencilState(DepthEnabling, 0);
		SetBlendState(DisableBlend, float4( 0.0f, 0.0f, 0.0f, 0.0f ), 0xFFFFFFFF);
    }
}


technique10 Parabolic10 <
	string Script = "Pass=p0;";
> {
    pass p0 <
	string Script = "Draw=geometry;";
    > {
        SetVertexShader( CompileShader( vs_4_0, std_VS() ) );
        SetGeometryShader( NULL );
        SetPixelShader( CompileShader( ps_4_0, parabMap_PS() ) );
                
        SetRasterizerState(DisableCulling);       
		SetDepthStencilState(DepthEnabling, 0);
		SetBlendState(DisableBlend, float4( 0.0f, 0.0f, 0.0f, 0.0f ), 0xFFFFFFFF);
    }
}


/////////////////////////////////////// eof //
